@@ -1,5 +1,5 @@
 /* ngettext - retrieve plural form string from message catalog and print it.
-   Copyright (C) 1995-1997, 2000-2007, 2012, 2018-2020 Free Software Foundation, Inc.
+   Copyright (C) 1995-1997, 2000-2007, 2012, 2018-2022 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include <locale.h>
 #include <errno.h>
 
+#include "attribute.h"
 #include "noreturn.h"
 #include "closeout.h"
 #include "error.h"
@@ -126,7 +127,7 @@ License GPLv3+: GNU GPL version 3 or later <%s>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-              "1995-1997, 2000-2020", "https://gnu.org/licenses/gpl.html");
+              "1995-1997, 2000-2022", "https://gnu.org/licenses/gpl.html");
       printf (_("Written by %s.\n"), proper_name ("Ulrich Drepper"));
       exit (EXIT_SUCCESS);
     }
@@ -143,7 +144,7 @@ There is NO WARRANTY, to the extent permitted by law.\n\
 
     case 4:
       domain = argv[optind++];
-      /* FALLTHROUGH */
+      FALLTHROUGH;
 
     case 3:
       break;
